@@ -180,6 +180,105 @@ class DocumentRouter:
         self.datum_page(bulk_datum_to_datum_page(doc))
 
 
+class SingleRunDocumentRouter(DocumentRouter):
+    """
+    A DocumentRouter intended to process events from exactly one run.
+    """
+    def __init__(self):
+        super().__init__()
+        self._start_doc = None
+        self._descriptors = dict()
+
+    def __call__(self, name, doc, validate=False):
+        """
+        Process a document.
+
+        Also, track of the start document and descriptor documents
+        passed to this SingleRunDocumentRouter in caches.
+
+        Parameters
+        ----------
+        name : string
+        doc : dict
+        validate : boolean
+            Apply jsonschema validation to the documents coming *out*. This is
+            False by default.
+
+        Returns
+        -------
+        name, output_doc : string, dict
+            The same name as what was passed in, and a doc that may be the same
+            instance as doc, a copy of doc, or a different dict altogether.
+        """
+        if name == 'start':
+            if self._start_doc is None:
+                self._start_doc = doc
+            else:
+                raise EventModelValueError(
+                    f'SingleRunDocumentRouter associated with start document {self._start_doc["uid"]} '
+                    f'received a second start document with uid {doc["uid"]}'
+                )
+        elif name == 'descriptor':
+            if doc['run_start'] == self._start_doc['uid']:
+                self._descriptors[doc['uid']] = doc
+            else:
+                raise EventModelValueError(
+                    f'SingleRunDocumentRouter associated with start document {self._start_doc["uid"]} '
+                    f'received a descriptor {doc["uid"]} associated with start document {doc["run_start"]}'
+                )
+        # Defer to superclass for dispatch/processing.
+        return super().__call__(name, doc, validate)
+
+    def get_start(self):
+        """Convenience method returning the start document for the associated run.
+
+        If no start document has been processed EventModelError will be raised.
+
+        Returns
+        -------
+        start document : dict
+        """
+        if self._start_doc is None:
+            raise EventModelError('SingleRunDocumentRouter has not processed a start document yet')
+
+        return self._start_doc
+
+    def get_descriptor(self, doc):
+        """Convenience method returning the descriptor associated with the specified document.
+
+        Parameters
+        ----------
+        doc : dict
+            event-model document
+
+        Returns
+        -------
+        descriptor document : dict
+        """
+        if 'descriptor' not in doc:
+            raise EventModelValueError(f'document is not associated with a descriptor:\n{doc}')
+        elif doc['descriptor'] not in self._descriptors:
+            raise EventModelValueError(
+                f'SingleRunDocumentRouter has not processed a descriptor with uid {doc["descriptor"]}'
+            )
+
+        return self._descriptors[doc['descriptor']]
+
+    def get_stream_name(self, doc):
+        """Convenience method returning the name of the stream for the specified document.
+
+        Parameters
+        ----------
+        doc : dict
+            event-model document
+
+        Returns
+        -------
+        stream name : str
+        """
+        return self.get_descriptor(doc).get('name')
+
+
 class HandlerRegistryView(collections.abc.Mapping):
     def __init__(self, handler_registry):
         self._handler_registry = handler_registry
@@ -386,7 +485,6 @@ class Filler(DocumentRouter):
                  descriptor_cache=None, inplace=None,
                  retry_intervals=(0.001, 0.002, 0.004, 0.008, 0.016, 0.032,
                                   0.064, 0.128, 0.256, 0.512, 1.024)):
-
         if inplace is None:
             self._inplace = True
             warnings.warn(
@@ -1046,9 +1144,13 @@ class RunRouter(DocumentRouter):
         # keyed on the RunStart UID referenced by that EventDescriptor.
         self._subfactory_cbs_by_start = defaultdict(list)
 
+        # Map RunStart UID to RunStart document. This is used to send
+        # RunStart documents to subfactory callbacks.
+        self._start_to_start_doc = dict()
+
         # Map RunStart UID to the list EventDescriptor. This is used to
         # facilitate efficient cleanup of the caches above.
-        self._descriptors = defaultdict(list)
+        self._start_to_descriptors = defaultdict(list)
 
         # Map EventDescriptor UID to RunStart UID. This is used for looking up
         # Fillers.
@@ -1068,8 +1170,9 @@ class RunRouter(DocumentRouter):
                 f"\n".join(f"    {factory}" for factory in self.factories) +
                 f"])")
 
-    def start(self, doc):
-        uid = doc['uid']
+    def start(self, start_doc):
+        uid = start_doc['uid']
+        self._start_to_start_doc[uid] = start_doc
         filler = self.filler_class(self.handler_registry,
                                    root_map=self.root_map,
                                    inplace=False)
@@ -1077,47 +1180,51 @@ class RunRouter(DocumentRouter):
         # No need to pass the document to filler
         # because Fillers do nothing with 'start'.
         for factory in self.factories:
-            callbacks, subfactories = factory('start', doc)
+            callbacks, subfactories = factory('start', start_doc)
             for callback in callbacks:
                 try:
-                    callback('start', doc)
+                    callback('start', start_doc)
                 except Exception as err:
                     warnings.warn(
                         DOCS_PASSED_IN_1_14_0_WARNING.format(
                             callback=callback, name='start', err=err))
+                    raise err
             self._factory_cbs_by_start[uid].extend(callbacks)
             self._subfactories[uid].extend(subfactories)
 
-    def descriptor(self, doc):
-        uid = doc['uid']
-        start_uid = doc['run_start']
-        self._fillers[start_uid].descriptor(doc)
+    def descriptor(self, descriptor_doc):
+        uid = descriptor_doc['uid']
+        start_uid = descriptor_doc['run_start']
+        self._fillers[start_uid].descriptor(descriptor_doc)
         # Apply all factory cbs for this run to this descriptor, and run them.
         factory_cbs = self._factory_cbs_by_start[start_uid]
         self._factory_cbs_by_descriptor[uid].extend(factory_cbs)
         for callback in factory_cbs:
-            callback('descriptor', doc)
-        # Let all the subfactories add any relavant callbacks.
+            callback('descriptor', descriptor_doc)
+        # Let all the subfactories add any relevant callbacks.
         for subfactory in self._subfactories[start_uid]:
-            callbacks = subfactory('descriptor', doc)
+            callbacks = subfactory('descriptor', descriptor_doc)
             self._subfactory_cbs_by_start[start_uid].extend(callbacks)
             self._subfactory_cbs_by_descriptor[uid].extend(callbacks)
             for callback in callbacks:
                 try:
-                    callback('start', doc)
+                    start_doc = self._start_to_start_doc[start_uid]
+                    callback('start', start_doc)
                 except Exception as err:
                     warnings.warn(
                         DOCS_PASSED_IN_1_14_0_WARNING.format(
                             callback=callback, name='start', err=err))
+                    raise err
                 try:
-                    callback('descriptor', doc)
+                    callback('descriptor', descriptor_doc)
                 except Exception as err:
                     warnings.warn(
                         DOCS_PASSED_IN_1_14_0_WARNING.format(
                             callback=callback, name='descriptor', err=err))
+                    raise err
         # Keep track of the RunStart UID -> [EventDescriptor UIDs] mapping for
         # purposes of cleanup in stop().
-        self._descriptors[start_uid].append(uid)
+        self._start_to_descriptors[start_uid].append(uid)
         # Keep track of the EventDescriptor UID -> RunStartUID for filling
         # purposes.
         self._descriptor_to_start[uid] = start_uid
@@ -1201,11 +1308,12 @@ class RunRouter(DocumentRouter):
         self._subfactories.pop(start_uid, None)
         self._factory_cbs_by_start.pop(start_uid, None)
         self._subfactory_cbs_by_start.pop(start_uid, None)
-        for descriptor_uid in self._descriptors.pop(start_uid, ()):
+        for descriptor_uid in self._start_to_descriptors.pop(start_uid, ()):
             self._descriptor_to_start.pop(descriptor_uid, None)
             self._factory_cbs_by_descriptor.pop(descriptor_uid, None)
             self._subfactory_cbs_by_descriptor.pop(descriptor_uid, None)
         self._resources.pop(start_uid, None)
+        self._start_to_start_doc.pop(start_uid, None)
 
 
 class EventModelError(Exception):
